--Assigns colors to individual chat channels.  Public domain.
declare('ChanColors', ChanColors or {})

ChanColors.version = 1.3

ChanColors.save_id = 42654226721

ChanColors.data = {}
ChanColors.original_OnEvent = chatreceiver.OnEvent



--Load the data out of the system notes
function ChanColors.Load()
	ChanColors.data = unspickle(LoadSystemNotes(ChanColors.save_id))
	if ChanColors.data == nil then
		--fresh, initialize
		ChanColors.data = {}
		ChanColors.data.channels = {}
		ChanColors.data.version = ChanColors.version
	elseif ChanColors.data.version == nil then
		--upgrade from versions < 1.3
		local tmp = ChanColors.data
		ChanColors.data = {}
		ChanColors.data.channels = tmp
		ChanColors.data.enabled = true
		ChanColors.data.version = ChanColors.version
	end

	--check version
	if ChanColors.data.version > ChanColors.version then
		-- oops, trying to load a newer version of the database
		-- TODO: disable saving or something?  Probably won't actually change enough in future to break compatibility.
		print('\127FFFF00WARNING:\127o  \127FFFFFFChanColors db version ' .. ChanColors.data.version .. ' is newer than plugin (' .. ChanColors.version .. ')\127o')
	elseif ChanColors.data.version < ChanColors.version then
		--upgrade from versions >= 1.3, if needed
		--currently, we just need to update the version number
		ChanColors.data.version = ChanColors.version
	end
end



--Save the data to the system notes
function ChanColors.Save()
	SaveSystemNotes(spickle(ChanColors.data), ChanColors.save_id)
end



--Intercept the original chatreceiver.OnEvent with our own, where we'll insert the colors
ChanColors.chatreceiver = {}
function ChanColors.chatreceiver:OnEvent(event, data)
	--data has msg, name, and faction, channelid (but not always!)
	if data ~= nil then
		if data.channelid ~= nil then
			local color = ChanColors.GetColor(data.channelid)
			if color ~= nil then
				data.msg = '\127' .. color .. data.msg .. '\127o'
			end
		end
	end
	ChanColors:original_OnEvent(event, data)
end



--Enable the plugin
function ChanColors.Enable()
	chatreceiver.OnEvent = ChanColors.chatreceiver.OnEvent
	ChanColors.data.enabled = true
	ChanColors.Save()
	print('\127FFFFFFChanColors enabled\127o')
end



--Disable the plugin
function ChanColors.Disable()
	chatreceiver.OnEvent = ChanColors.original_OnEvent
	ChanColors.data.enabled = false
	ChanColors.Save()
	print('\127FFFFFFChanColors disabled\127o')
end



--Toggle the plugin
function ChanColors.Toggle()
	if ChanColors.data.enabled then
		ChanColors.Disable()
	else
		ChanColors.Enable()
	end
end



--Get the color assigned to a channel, or nil if there isn't one
function ChanColors.GetColor(channelid)
	channelid = tonumber(channelid)
	if ChanColors.data.channels == nil then return nil end
	if ChanColors.data.channels[channelid] == nil then return nil end
	return(ChanColors.data.channels[channelid])
end



--Set the color for a channel, using hex colors (# is optional)
function ChanColors.SetColor(channelid, color)
	--clean up variables
	color = string.gsub(color, '#', '')
	channelid = tonumber(channelid)
	--set and save the data
	ChanColors.data.channels[channelid] = color
	ChanColors.Save()
	--demo it!
	ChanColors.Demo(channelid, color)
end



--Unset a color, causing channel to use default
function ChanColors.UnsetColor(channelid)
	channelid = tonumber(channelid)
	ChanColors.data.channels[channelid] = nil
	ChanColors.Save()
	print('\127FFFFFF[' .. channelid .. '] cleared\127o')
end



--Delete all colors - use with care ;)
function ChanColors.Clear()
	ChanColors.data.channels = {}
	ChanColors.Save()
	print('\127FFFFFFAll channels cleared\127o')
end



--List all channels with custom colors, along with samples
function ChanColors.List()
	for channelid, color in pairs(ChanColors.data.channels) do
		ChanColors.Demo(channelid, color)
	end
end



--Show what the color will look like
function ChanColors.Demo(channelid, color)
	channelid = tonumber(channelid)
	print('\127FFFFFF[' .. channelid .. '] : ' .. color .. ' : ' .. '\127o\127' .. color .. 'Sample text.\127o')
end



--Initialize when the player enters the game
function ChanColors.EnteredGame(event, data)
	ChanColors.Load()
	if ChanColors.data.enabled then
		ChanColors.Enable()
	end
end



--Display usage
function ChanColors.Usage()
	print('\127FFFFFFChanColors v' .. ChanColors.version .. '\127o')
	print('\127FFFFFFset\127o\127A0A0A0 channelid color\127o')
	print('\127FFFFFFunset\127o\127A0A0A0 channelid\127o')
	print('\127FFFFFFclearall\127o')
	print('\127FFFFFFlist\127o')
	print('\127FFFFFFenable|disable|toggle\127o')
	print('\127FFFFFFEXAMPLE:\127o  \127A0A0A0/ChanColors set 11 AA0000\127o')
end



--Process user commands
function ChanColors.Command(_,data)
	if (data == nil) then
		ChanColors.Usage()
	elseif (data[1] == "set") and data[2] ~= nil and data[3] ~= nil then
		ChanColors.SetColor(data[2], data[3])
	elseif (data[1] == "unset") and data[2] ~= nil then
		ChanColors.UnsetColor(data[2])
	elseif (data[1] == "clearall") then
		ChanColors.Clear()
	elseif (data[1] == "list") then
		ChanColors.List()
	elseif (data[1] == "enable") then
		ChanColors.Enable()
	elseif (data[1] == "disable") then
		ChanColors.Disable()
	elseif (data[1] == "toggle") then
		ChanColors.Toggle()
	else
		print('\127FF4040Error\127o')
		ChanColors.Usage()
	end
end



RegisterUserCommand("ChanColors",ChanColors.Command)
RegisterEvent(ChanColors.EnteredGame, 'PLAYER_ENTERED_GAME')



